from hysop.tools.decorators import debug
from hysop.backend.device.opencl.opencl_operator import OpenClOperator, op_apply
from hysop.operator.base.integrate import IntegrateBase

class OpenClIntegrate(IntegrateBase, OpenClOperator):

    @debug
    def __init__(self, **kwds):
        super(OpenClIntegrate, self).__init__(**kwds)

    @debug
    def get_field_requirements(self):
        # force 0 ghosts for the reduction (pyopencl reduction kernel)
        requirements = super(OpenClIntegrate, self).get_field_requirements()
        topo, req = requirements.get_input_requirement(self.field)
        req.max_ghosts = (0,)*self.field.dim
        return requirements

    @debug
    def setup(self, work):
        super(OpenClIntegrate, self).setup(work)
        self.sum_kernels = tuple(self.dF.backend.sum(a=self.dF.data[i],
                                 build_kernel_launcher=True, async=True)
                                 for i in xrange(self.dF.nb_components))

    @op_apply
    def apply(self, **kwds):
        queue = self.cl_env.default_queue
        evts = ()
        for knl in self.sum_kernels:
            evt = knl(queue=queue)
            evts += (evt,)
        value = self.parameter._value.copy()
        for (i,evt) in enumerate(evts):
            evt.wait()
            Pi = self.sum_kernels[i].out.get()[0]
            if (self.scaling_coeff[i] is None):
                self.scaling_coeff[i] = 1.0 / Pi
            value[i] = self.scaling_coeff[i] * Pi

        # compute value from all processes
        self.parameter.value = self._collect(value)
