import numpy as np

try:
    import hptt
    HAS_HPTT=True
    # required version is: https://gitlab.com/keckj/hptt
except ImportError:
    hptt = None
    HAS_HPTT=False
    import warnings
    from hysop.tools.warning import HysopPerformanceWarning
    msg='Failed to import HPTT module, falling back to slow numpy transpose. Required version is available at https://gitlab.com/keckj/hptt.'
    warnings.warn(msg, HysopPerformanceWarning)

def array_share_data(a,b):
    abeg, aend = np.byte_bounds(a)
    bbeg, bend = np.byte_bounds(b)
    beg, end = max(abeg, bbeg), min(aend, bend)
    return (end-beg > 0)

if HAS_HPTT:
    def can_exec_hptt(src, dst):
        if (src is dst):
            return False
        if (src.dtype != dst.dtype):
            return False
        if src.dtype not in (np.float32, np.float64, np.complex64, np.complex128):
            return False
        if src.flags['C_CONTIGUOUS'] != dst.flags['C_CONTIGUOUS']:
            return False
        if src.flags['F_CONTIGUOUS'] != dst.flags['F_CONTIGUOUS']:
            return False
        if not (src.flags['C_CONTIGUOUS'] ^ src.flags['F_CONTIGUOUS']):
            return False
        return not array_share_data(src, dst)
else:
    def can_exec_hptt(src, dst):
        return False
